<?php

namespace ModelExporter;

use PHPUnit\Util\Getopt;

class Command
{
    /**
     * @var array $options option[0] - parameters with '--' prefix
     *                     option[1] - parameters without prefix
     */
    protected $options = [];

    protected $defaultDir = '/schemas/';

    /**
     * @var array
     */
    protected $longOptions = [
        'saveDir=' => null,
        'pathFile='    => null,
        'pathDir='  => null
    ];

    public static function main(bool $exit = true): int
    {
        $command = new static;

        return $command->run($_SERVER['argv'], $exit);
    }

    public function run(array $argv, bool $exit = true): int
    {
        try {
            $this->options = Getopt::getopt(
                $argv,
                '',
                \array_keys($this->longOptions)
            );
        } catch (\Exception $e) {
            exit($e->getMessage());
        }

        $pathFile = $pathDir = $yamlStructure = null;
        $saveDir = "";
        foreach ($this->options[0] as $option) {
            if ($option[0] == "--pathFile") {
                $pathFile = $option[1];
            }
            if ($option[0] == "--saveDir") {
                $saveDir = $option[1];
            }
            if ($option[0] == "--pathDir") {
                $pathDir = substr($option[1], -1) === '/' ? $option[1] : $option[1] . '/';
            }
        }

        $generator = new MEGenerator();
        $file = new File();
        if ($pathFile) {
            if (!$data = $file->getData($pathFile)) {
                print_r("Path is not found!\nPath: $pathFile\n");
                return false;
            }

            $nameEntity = basename(substr($pathFile, 0, strpos($pathFile, '.')));
            $yamlStructure = $generator->generate([$nameEntity => $data]);
        } elseif ($pathDir) {
            if ($handle = opendir($pathDir)) {
                $data = [];
                while (false !== ($existFile = readdir($handle))) {
                    if (!empty($existFile) && ($existFile != '..') && ($existFile != '.')) {
                        $nameEntity = basename(substr($existFile, 0, strpos($existFile, '.')));
                        $data[$nameEntity] = $file->getData($pathDir . $existFile);
                    }
                }
                closedir($handle);
                $yamlStructure = $generator->generate($data);
            }
        }

        $saveDir = is_dir($saveDir) ? $saveDir : dirname(__FILE__, 2) . $this->defaultDir;
        if ($yamlStructure) {
            $yaml = $generator->annotations . \Symfony\Component\Yaml\Yaml::dump($yamlStructure, 9);
            $yaml = str_replace("'(1..*)'", "(1..*)", str_replace("'~'", "~", $yaml));
            $file->save($yaml, $saveDir);

            return true;
        }

        return false;
    }
}
