<?php

namespace ModelExporter;

class MEGenerator
{
    public $annotations = "annotationGenerators:\n    - ApiPlatform\SchemaGenerator\AnnotationGenerator\PhpDocAnnotationGenerator\n    - ApiPlatform\SchemaGenerator\AnnotationGenerator\DoctrineOrmAnnotationGenerator\n    - ApiPlatform\SchemaGenerator\AnnotationGenerator\ConstraintAnnotationGenerator\n";

    protected $namespaces = [
        "entity" => "Entity"
    ];

    public function generate(?array $setData)
    {
        if (!$setData) {
            return null;
        }

        $result = [];
        foreach ($setData as $nameEntity => $data) {
            list($columns, $entities) = $this->getFields($data);
            $selectedPostTypes = $this->generateOneEntity($nameEntity, $columns, $entities);
            $result = array_merge($result, $selectedPostTypes);
        }

        $array = [
            "namespaces" => $this->namespaces,
            "types" => $result
        ];

        return $array;
    }

    protected function getFields(string $data)
    {
        $columns = $entities = [];

        $pattern = '/(?<=hasColumn\()[a-z.0-9\'=>\/,\n\t()_ ]*(?=\))/i';
        preg_match_all($pattern, $data, $matches);
        foreach ($matches[0] as $match) {
            $patternField = "(\'.*?\')";
            $field = "";
            preg_match($patternField, $match, $field);
            if ($field = str_replace('\'', '', $field[0])) {
                if (substr($field, -3) === '_id') {
                    $entities[] = $field;
                } else {
                    $columns[] = $field;
                }
            }
        }

        return [$columns, $entities];
    }

    protected function generateOneEntity($type, $columns, $entities)
    {
        $selectedTypes[$type] = [
            'properties' => []
        ];

        if ($columns) {
            foreach ($columns as $column) {
                $selectedTypes[$type]['properties'] = array_merge($selectedTypes[$type]['properties'], [$column => '~']);
            }
        }

        if ($entities) {
            foreach ($entities as $entity) {
                $selectedTypes[$type]['properties'][$entity] = [
                    'range' => $entity,
                    'cardinality' => "(1..*)",
                ];
            }
        }

        return $selectedTypes;
    }
}
